/*
    u200mon -- the WiMax monitor and logger
    Copyright (C) 2008 Anthony A Z <dci@cpp.in>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdio.h>
#include <conio.h>
#include <windows.h>

#include "u200.h"
#include "nmea.h"
#include "ping.h"


char * u200_states [ ] = { "INIT", "SYNC", "NEGO", "NORMAL", "SLEEP", "IDLE", "HHO", "FBSS", "RESET", "RESERVED", "UNDEFINED", "BE", "NRTPS", "RTPS", "ERTPS", "UGS", "INITIAL_RNG", "BASIC", "PRIMARY", "SECONDARY", "MULTICAST", "NORMAL_MULTICAST", "SLEEP_MULTICAST", "IDLE_MULTICAST", "FRAG_BROADCAST", "BROADCAST", "MANAGEMENT", "TRANSPORT" };

char  u200_state [ 32 ] = "UNDEFINED";
float u200_cinr = 0;
short u200_rssi = 0;
unsigned short u200_txpw = 0;
unsigned char u200_bsid [ 6 ];

void u200_cb ( unsigned int state, float cinr, short rssi, unsigned short tx_power, unsigned char * bsid )
{
	strcpy ( u200_state, u200_states [ ( state < ( sizeof ( u200_states ) / sizeof ( * u200_states ) ) ) ? state : 10 ] );
	u200_cinr = cinr;
	u200_rssi = rssi;
	u200_txpw = tx_power;
	memcpy ( u200_bsid, bsid, sizeof ( u200_bsid ) );
}


double nmea_lat  = 0;
double nmea_long = 0;

void nmea_cb ( double latitude, double longitude )
{
	nmea_lat  = latitude;
	nmea_long = longitude;
}

int ping_rtt = 0;

void ping_cb ( int rtt )
{
	ping_rtt = rtt;
}


int wmain ( int argc, wchar_t * argv [ ] )
{
	wchar_t * nmeaport = NULL;
	wchar_t * datafile = NULL;
	wchar_t * pinghost = NULL;
	int       updatetm = 5;
	int       i;
	FILE *    dfhandle = NULL;
	/* */
	printf ( "u200mon 0.1b\n(c) 2008 Anthony A Z <dci@cpp.in>\nhttp://cpp.in/dev\n" );
	/* */
	if ( 1 == argc ) printf ( " Use -h for help.\n" );
	if ( 2 == argc && !_wcsicmp ( L"-h", argv [ 1 ] ) )
	{
		printf ( " usage: u200mon [-n COMx:] [-o file.csv] [-i number] [-p host]\n   -n: Specifies the com-port to read NMEA messages from.\n   -o: Specifies the file to write data to.\n   -i: Specifies interval of data update (seconds).\n   -p: Specifies the host to ping to keep the connection alive\n" );
		return 0;
	}
	else
	{
		for ( i = 0; i < argc; ++i )
		{
			if ( !_wcsicmp ( L"-n", argv [ i ] ) ) nmeaport = argv [ ++i ];
			else if ( !_wcsicmp ( L"-o", argv [ i ] ) ) datafile = argv [ ++i ];
			else if ( !_wcsicmp ( L"-i", argv [ i ] ) ) updatetm = _wtoi ( argv [ ++i ] );
			else if ( !_wcsicmp ( L"-p", argv [ i ] ) ) pinghost = argv [ ++i ];
		}
	}
	/* */
	i = u200_start ( u200_cb );
	if ( 0 != i )
	{
		printf ( "Unable to connect to u200 device: %08X %08X.\n", i, GetLastError ( ) );
		return 1;
	}
	/* */
	if ( nmeaport )
	{
		i = nmea_start ( nmeaport, nmea_cb );
		if ( 0 != i )
		{
			wprintf ( L"Unable to connect to NMEA source via '%s': %08X %08X\n", nmeaport, i, GetLastError ( ) );
			u200_stop ( );
			return 2;
		}
	}
	else printf ( "No NMEA source specified, GPS data will not be available.\n" );
	/* */
	if ( datafile )
	{
		dfhandle = _wfopen ( datafile, L"a+" );
		if ( !dfhandle )
		{
			printf ( "Cannot open '%s' for writing: %08X\n", datafile, GetLastError ( ) );
			nmea_stop ( );
			u200_stop ( );
			return 3;
		}
	}
	/* */
	if ( pinghost ) ping_start ( pinghost, ping_cb );
	printf ( "Hold esc to exit\n" );
	/* */
	i = 0;
	while ( 0 == ( GetAsyncKeyState ( VK_ESCAPE ) & 0x80000000 ) )
	{
		if ( updatetm * 4 <= i++ )
		{
			printf ( "%.6f %.6f %s %.1f %d %d %02X:%02X:%02X:%02X:%02X:%02X %i          \r", nmea_lat, nmea_long, u200_state, u200_cinr, u200_rssi, u200_txpw, u200_bsid [ 0 ], u200_bsid [ 1 ], u200_bsid [ 2 ], u200_bsid [ 3 ], u200_bsid [ 4 ], u200_bsid [ 5 ], ping_rtt );
			if ( dfhandle )
			{
				SYSTEMTIME st;
				GetLocalTime ( &st );
				fprintf ( dfhandle, "%02d/%02d/%04d %d:%02d:%02d,%.6f,%.6f,%s,%.1f,%d,%d,%02X:%02X:%02X:%02X:%02X:%02X,%i\n", st.wDay, st.wMonth, st.wYear, st.wHour, st.wMinute, st.wSecond, nmea_lat, nmea_long, u200_state, u200_cinr, u200_rssi, u200_txpw, u200_bsid [ 0 ], u200_bsid [ 1 ], u200_bsid [ 2 ], u200_bsid [ 3 ], u200_bsid [ 4 ], u200_bsid [ 5 ], ping_rtt );
				fflush ( dfhandle );
			}
			i = 0;
		}
		Sleep ( 250 );
	}
	/* */
	printf ( "\nCleaning up...\n" );
	if ( dfhandle ) fclose ( dfhandle );
	ping_stop ( );
	nmea_stop ( );
	u200_stop ( );
	return 0;
}
